#!/bin/python
# vim:et:ts=4

import settings
from IRCClient import *
from AuthDB import *

class AuthBot(LogAllMixin, IRCClient):
    def __init__(self, server, port, nick, channel, authdb):
        self.channel = channel
        self.authdb = authdb
        IRCClient.__init__(self, server, port)
        self.connect(nick, "AuthBot")
        self.join(channel)
        self.users = {}

    def handle_invite(self, source, to, channel):
        if to == self.get_nick() and channel == self.channel:
             self.join(channel)

    def handle_join(self, fullnick, channel):
        nick = getnick(fullnick)
        mynick = self.get_nick()
        self.users[nick.lower()] = None
        if channel == self.channel and nick != mynick:
            self.say(nick, "Authentication required in this channel. If you have an account, \"/msg " + mynick + " ident username password\". If you do not have an account, visit " + settings.CGI_URL + " in a web browser.")

    def handle_part(self, nick, channel, reason):
        nick = getnick(nick).lower()
        if self.users.has_key(nick):
            del self.users[nick]

    def handle_quit(self, nick, reason):
        nick = getnick(nick).lower()
        if self.users.has_key(nick):
            del self.users[nick]

    def handle_nick(self, old, new):
        (old, new) = (getnick(old).lower(), getnick(new).lower())
        if self.users.has_key(old):
            self.users[new] = self.users[old]
            del self.users[old]
        else:
            self.users[new] = None

    def handle_say(self, source, to, message):
        if to != self.get_nick(): return
        nick = getnick(source)

        l = message.strip().lower().split(" ")
        if len(l) < 1: 
            self.say(nick, "Empty command.")
            return

        if l[0] == "ident":
            if len(l) == 2:
                (dummy, password) = l
                username = nick
            elif len(l) == 3:
                (dummy, username, password) = l
            else:
                self.say(nick, "Usage: \"/msg " + self.get_nick() + " ident [username] password\". username is assumed to be " + nick + " if not supplied.")
                return

            modes = db.login(username, password)
            if modes is None:
                self.say(nick, "Unable to ident you.")
            else:
                self.say(nick, "Ident OK, " + username)
                self.usermode(self.channel, nick, modes)
                self.users[nick.lower()] = username

        elif l[0] == "whois":
            if len(l) != 2:
                self.say(nick, "Usage: \"whois username\".")
            else:
                username = l[1].lower()
                if not self.users.has_key(username):
                    self.say(nick, "Unknown user.")
                elif self.users[username] is None:
                    self.say(nick, username + " has not authenticated.")
                else:
                    self.say(nick, username + " is " + self.users[username])

        else:
            self.say(nick, "I'm a bot. Available commands: \"whois\", \"ident\". See " + settings.CGI_URL + " for more information.")
            
if __name__ == "__main__":
    db = AuthDB(settings.DB_PATH)
    bot = AuthBot(settings.SERVER, settings.PORT,
                  settings.NICK, settings.CHANNEL, db)
    bot.mainloop()

